/******************************************************************************
 * 
 * $Source: /src/rcs/Cats/XSystem/pex/mbx/x_pex_interaction.c,v $
 * $Date: 93/12/30 13:53:30 $
 * $Revision: 1.4 $
 *                                                                            
 * Description:   
 *	This tests exercised mixing X and PEX rendering to MBX buffers.
 *
 * Writtien by: Ken Tidwell
 ******************************************************************************/


/******************************************************************************/
/*                                                                            */
/* Include Files                                                              */
/*                                                                            */
/******************************************************************************/

/* ------ system and platform files ------------------------------------------*/
#include <X11/PEX5/PEXlib.h>
#include <X11/PEX5/PEXHPlib.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/extensions/multibuf.h>
#include <stdio.h>
#include <math.h>

/* ------ program specific ---------------------------------------------------*/
#include <lut.h>
#include <misc.h>
#include "pexutcmap.h"
#include "common.h"

#include "common.c"

void testproc(Display *dpy, Window window, int cur_image);

void inquire_test_params(char *test_name, 
			 int  *num_images, 
			 int  *supported_modes, 
			 char *win_title)
{
    strcpy(test_name, "mbx_centerline");
    *num_images      = 5;
    *supported_modes = IMM_MODE;

    strcpy(win_title, "MBX Centerline");
}


/******************************************************************************/
void
main( argc, argv )
	int  argc;
	char **argv;
/******************************************************************************/
{
    Display		       *Dpy;
    Window			Win;
    Drawable			drawable_to_use;
    int                         Screen=0;
    XEvent			Event;
    XColor			BackGroundColor;
    XStandardColormap		ColorMapInfo;
    XVisualInfo			VisualInfo;
    PEXUtVisualCriteria         Criteria;
    unsigned int                UnmetCriteria;
    PEXUtWindowSpecification    WindowInfo;
    Atom                        StdPropAtom;
    PEXColorApproxEntry		CapxInfo;
    PEXRenderer			Renderer;
    PEXRendererAttributes	RAttrs;
    PEXExtensionInfo	       *pexinfo;
    int				Done = 0,
				CubeType = WIREFRAME, /* Default WIREFRAME    */
                                ColorFlag = REDLINES; /* Default RED Cube     */
    double                      DeltaVal=WIREFRAMEDELTA, /* Rotation increment*/
				Angle=0;              /* Rotation angle       */
    XSizeHints 			Hints;
    int 			Result;
    GC				gc;
    XGCValues			gc_vals;
    /*
    ** Variables for tracking MBX operations. We do not support more than
    ** 2 buffers currently. I am writing this test framework to be extensible
    ** to multiple buffers in the future. -KT
    */
    Multibuffer			buffers[4], displayed_buffer, render_buffer;
    int				num_buffers;
    
    /*
    ** Variables for finding out about MBX version.
    */
    int 			mbx_event_base, mbx_error_base;
    int 			server_major_version, server_minor_version;
    Status			status;
    XColor			xcolor;
    char			err[80];

    /** Other Variables for susPEX
    */
    int			pexmode = 0;
    int image_depth = 0;


    /*
    ** Open a display and initialize PEX.
    */
    
    Dpy = init_pex( argv, &pexinfo );
    if ( Dpy == (Display *)NULL )
    {
        exit(1);
    }

    /*
     * Set up error handler
     */
    _hppex_test_init_error_handler();

    /*  Handle window geometry string if present */
    parse_window_geometry(argc, argv);

    /* Handle other arguments. */
    if (parse_options (argc, argv, &pexmode, &image_depth))
	exit (1);


    /*
    ** Set up properties of the window
    */
    Screen = DefaultScreen(Dpy);
    Hints.x = XPOS;
    Hints.y = YPOS;
    Hints.width = WIDTH;
    Hints.height = HEIGHT;
    Hints.flags = (USSize|USPosition);

    Criteria.hard_criteria_mask = PEXUtMinColors | 
				  PEXUtDoubleBufferingCapability;
    Criteria.soft_criteria_mask = PEXUtStandardColormapProperty;
    Criteria.min_colors = 2;
    Criteria.standard_colormap_property = True;
    Criteria.double_buffering_capability = PEXUtDbufferPEX;

    WindowInfo.attr_mask = 0;
    WindowInfo.title = "Double-Buffering";
    WindowInfo.size_hints = Hints;
    WindowInfo.parent = RootWindow (Dpy, Screen);
    WindowInfo.border_width = 0;
    WindowInfo.background_color_name = "black";
    WindowInfo.border_color_name = "blue";

    /*
    ** Create window and colormap
    */
    Result = PEXUtCreateWindowAndColormap (Dpy, Screen, &Criteria,
					    &WindowInfo, &Win,
					    &VisualInfo, &ColorMapInfo,
					    &CapxInfo,
					    &UnmetCriteria, &StdPropAtom,
					    &BackGroundColor);

    if ( Win == None )
    {
	fprintf (stderr, "Could not create window and colormap!\n");
	exit (2);
    }

    XSetWindowColormap(Dpy, Win, ColorMapInfo.colormap);
    XInstallColormap(Dpy, ColorMapInfo.colormap);
    XSync(Dpy, 0);

    /*
    ** Create GC for X renderering
    */

    /*
    ** Set the foreground to white in the colormap ramp.
    */

    xcolor.red = 0xffff;
    xcolor.green = 0xffff;
    xcolor.blue = 0xffff;

    XAllocColor (Dpy, ColorMapInfo.colormap, &xcolor);

    gc_vals.foreground = xcolor.pixel;

    /*
    ** Set the background to black in the colormap ramp.
    */

    xcolor.red = 0x0;
    xcolor.green = 0x0;
    xcolor.blue = 0x0;

    XAllocColor (Dpy, ColorMapInfo.colormap, &xcolor);

    gc_vals.background = xcolor.pixel;

    gc = XCreateGC (Dpy, Win, GCForeground | GCBackground, &gc_vals);

    /*
    ** Create renderer.
    */
    Renderer = setup_renderer( Dpy, Win, &CapxInfo, &RAttrs );

    /*
    ** Set the renderer to clear image at BeginRendering.
    ** Turn Z buffer on.
    */
    RAttrs.clear_image = True;
    RAttrs.hlhsr_mode = PEXHLHSRZBuffer;
    PEXChangeRenderer (Dpy, Renderer, PEXRAHLHSRMode | PEXRAClearImage,
		       &RAttrs);


    /**************************************************************************
    ** Set up MBX buffers
    ***************************************************************************
    */

    /*
    ** STEP 1
    **
    ** Setup and configure MBX buffering.
    */

    /* 
    ** 1A
    ** 
    ** See if MBX is there.
    */
    if (! XmbufQueryExtension (Dpy, &mbx_event_base, &mbx_error_base))
    {
	fprintf(stderr,"MBX extension not found!\n");
	exit (3);
    }
   
    /*
    ** 1B
    **
    ** See what version of MBX is there. We want to use 3.2 protocol, which 
    ** corresponds to server version 1.0, which is what we check for.
    */
    status = XmbufGetVersion (Dpy, &server_major_version, 
			      &server_minor_version);
    if ((! status) || (server_major_version != 1) || 
	(server_minor_version != 0))
    {
	fprintf(stderr,"MBX version not compatible!\n");
	exit (4);
    }

    /*
    ** 1C
    **
    ** Create  buffers.
    **
    */
    num_buffers =  XmbufCreateBuffers (Dpy, Win, 2, 
				       MultibufferUpdateActionUndefined,
				       MultibufferUpdateHintFrequent,
				       buffers);
    if (num_buffers != 2)
    {
	fprintf(stderr,"Number of buffers requested not allocated!\n");
	exit (5);
    }

    /*
    ** Displayed buffer is first buffer in buffers list returned. Render
    ** buffer will be determined in render portion of test.
    */
    displayed_buffer = 0;

   
    /*
    ** Ensure the output buffer is flushed (without discarding events).
    */
    XSync ( Dpy, False );

    /*
    **
    ** Create the cube and set the correct view
    */
    SetColors ( Dpy, RAttrs.color_table);
    SetView( Dpy, RAttrs.view_table );

    /*************************************************************************
    **
    ** TEST CASES BEGIN HERE
    **
    *************************************************************************/

    /*
    ** 
    ** TEST 15 
    **
    */
    describe_test("TEST 15: Mix X and PEX rendering\n\n");


    /*
    ** Select render_buffer to be back
    */
    render_buffer = 1; /* Back buffer */

    /*
    ** Redraw the cube RED and solid.
    */
    redraw( Dpy, buffers[render_buffer], Renderer, (Angle*M_PI)/180.0, 
	    REDLINES, SOLID);

    /*
    ** Draw border and label.
    */
    XDrawRectangle (Dpy, buffers[render_buffer], gc,  4, 4,
		    WIDTH-8, HEIGHT-8);
    XDrawString (Dpy, buffers[render_buffer], gc, 8, HEIGHT -12,
		 "Label for Test 14", 17);
    

    /*
    ** Check results 1. 
    */
    describe_test ("Step 1: Window should be blank. \n");
    describe_test ("Press return to continue\n\n");
    testproc(Dpy, Win, 1);
    
    /*
    ** Swap buffers 
    */
    XmbufDisplayBuffers (Dpy, 1, &buffers[render_buffer], 0, 0);
    XSync ( Dpy, False );  /* Push request over wire so we can check image. */
    displayed_buffer = render_buffer;
    render_buffer = !render_buffer;

    /*
    ** Check results 2. 
    */
    describe_test ("Step 2: Red cube with border and label should be visible\n");
    describe_test ("Press return to continue\n\n");
    testproc(Dpy, Win, 2);

    /*
    ** Redraw the cube Green and wireframe at a slight angle.
    */

    Angle += 60;
    redraw( Dpy, buffers[render_buffer], Renderer, (Angle*M_PI)/180.0, 
	    GREENLINES, WIREFRAME);

    /*
    ** Draw border and label.
    */
    XDrawRectangle (Dpy, buffers[render_buffer], gc,  4, 4,
		    WIDTH-8, HEIGHT-8);
    XDrawString (Dpy, buffers[render_buffer], gc, 8, HEIGHT -12,
		 "Label for Test 14", 17);

    /*
    ** Check results 3. 
    */
    describe_test ("Step 3: Red cube should still be visible with border & label.\n");
    describe_test ("Press return to continue\n\n");
    testproc(Dpy, Win, 3);


    /*
    ** Swap buffers
    */
    XmbufDisplayBuffers (Dpy, 1, &buffers[render_buffer], 0, 0);
    XSync ( Dpy, False );  /* Push request over wire so we can check image. */
    displayed_buffer = render_buffer;
    render_buffer = !render_buffer;

    /*
    ** Check results 4. 
    */
    describe_test ("Step 4: Green wire cube with border & label should be visible.\n");
    describe_test ("Press return to continue\n\n");
    testproc(Dpy, Win, 4);

    /*
    ** Clear window
    */
    PEXBeginRendering (Dpy, Win, Renderer);
    PEXEndRendering (Dpy, Renderer, True);


    /*
    ** Draw border and label in front.
    */
    XDrawRectangle (Dpy, buffers[displayed_buffer] /*Win*/, gc, 4, 4,
		    WIDTH-8, HEIGHT-8);
    XDrawString (Dpy, buffers[displayed_buffer] /*Win*/, gc, 8, HEIGHT -12,
		 "Label for Test 14", 17);
    XSync (Dpy, False);

    /*
    ** Check results 5. 
    */
    describe_test ("Step 5: Window should be blank with only border and label\n");
    describe_test ("Press return to continue\n\n");
    testproc(Dpy, Win, 5);

    /*
    ** Draw border and label in back.
    */
    XDrawRectangle (Dpy, buffers[render_buffer], gc, 4, 4,
		    WIDTH-8, HEIGHT-8);
    XDrawString (Dpy, buffers[render_buffer], gc, 8, HEIGHT -12,
		 "Label for Test 14", 17);
    /*
    ** Redraw the cube RED and solid.
    */
    redraw( Dpy, buffers[render_buffer], Renderer, (Angle*M_PI)/180.0, 
	    REDLINES, SOLID);

    /*
    ** Swap buffers
    */
    XmbufDisplayBuffers (Dpy, 1, &buffers[render_buffer], 0, 0);
    XSync ( Dpy, False );  /* Push request over wire so we can check image. */
    displayed_buffer = render_buffer;
    render_buffer = !render_buffer;

    /*
    ** Check results 6. 
    */
    describe_test ("Step 6: Red cube should be visible without border & label\n");
    describe_test ("Press return to continue\n\n");
    testproc(Dpy, Win, 6);
    
    /*
    ** Redraw the cube GREEN and WIRE
    */
    redraw( Dpy, buffers[render_buffer], Renderer, (Angle*M_PI)/180.0, 
	    GREENLINES, WIREFRAME);

    /*
    ** Draw border and label in back.
    */
    XDrawRectangle (Dpy, buffers[render_buffer], gc, 4, 4,
		    WIDTH-8, HEIGHT-8);
    XDrawString (Dpy, buffers[render_buffer], gc, 8, HEIGHT -12,
		 "Label for Test 14", 17);

    /*
    ** Swap buffers
    */
    XmbufDisplayBuffers (Dpy, 1, &buffers[render_buffer], 0, 0);
    XSync ( Dpy, False );  /* Push request over wire so we can check image. */
    displayed_buffer = render_buffer;
    render_buffer = !render_buffer;

    /*
    ** Check results 7. 
    */
    describe_test ("Step 7: Green cube should be visible with border & label\n");
    describe_test ("Press return to continue\n\n");
    testproc(Dpy, Win, 7);
    

    /*
     * Test is complete. Quit.
     */

    XCloseDisplay( Dpy );
}

void testproc(Display *dpy, Window window, int cur_image)
{
    char image_name[80], test_name[80];

    sprintf(image_name,"mbx_centerline_%d", cur_image);
    sprintf(test_name, "MBX Centerline #%d", cur_image);
    image_testproc(image_name, test_name, img_desc,
		      dpy,window, 0, 0, 
		      glob_window_width, glob_window_height);
}


